@@ -5,7 +5,7 @@ from itypes import rtypes
 import ops
 _delay_import_(globals(),"utils","util")
 _delay_import_(globals(),"representor")
-_delay_import_(globals(),"wrappers","wrapper_py")
+_delay_import_(globals(),"wrappers","python")
 _delay_import_(globals(),"itypes", "casts","dimpaths","dimensions")
 
 class FuncSignature(object):
@@ -24,13 +24,6 @@ class Param(object):
         self.type = type
         self.default = default
 
-    def getStateMask(self):
-        if(self.type == rtypes.unknown):
-            return RS_SLICES_KNOWN
-        else:
-            return RS_ALL_KNOWN
-    state_mask = property(fget=getStateMask)
-
     def __str__(self):
        if(self.type is None):
            res = self.name
@@ -119,24 +112,22 @@ class Func(object):
 class UnaryFuncOp(repops.UnaryOpRep, Func):
     def __init__(self, source, *params, **kwargs):
         if(not isinstance(source, representor.Representor)):
-            source = wrapper_py.Rep(source)
+            source = python.Rep(source)
         repops.UnaryOpRep.__init__(self,source, *params,**kwargs)
 
 class UnaryFuncElemOp(UnaryFuncOp):
     def _process(self, source, **kwargs):
-        state = source._state
-        if not state & RS_TYPES_KNOWN:
+        if not source._typesKnown():
             return
 
         nslices = []
         for slice in source._slices:
             slice = self._prepareSlice(slice)
             sig, nkwargs, outparam = self._findSignature(slice=slice,**kwargs)
-            state &= outparam.state_mask
             s = ops.UnaryFuncElemOp(self.__class__.__name__, sig, outparam, **nkwargs)
             s = self._finishSlice(s)
             nslices.append(s)
-        return self._initialize(tuple(nslices),state)
+        return self._initialize(tuple(nslices))
 
     def _prepareSlice(self,slice):
         return slice
@@ -148,14 +139,13 @@ class UnaryFuncDimOp(UnaryFuncOp):
     _slicecls = ops.UnaryFuncSeqOp
 
     def _process(self, source, dim=None, **kwargs):
-        state = source._state
-        if not state & RS_TYPES_KNOWN:
+        if not source._typesKnown():
             return
-        nslices,state = self._apply(source._slices, dim, source._state, **kwargs)
-        return self._initialize(tuple(nslices),state)
+        nslices = self._apply(source._slices, dim, **kwargs)
+        return self._initialize(tuple(nslices))
 
     @classmethod
-    def _apply(cls, xslices, dim, state=None, **kwargs):
+    def _apply(cls, xslices, dim,  **kwargs):
         nslices = []
         found = False
         for slice in xslices:
@@ -167,8 +157,6 @@ class UnaryFuncDimOp(UnaryFuncOp):
             for lastpos in lastposs:
                 packdepth = len(slice.dims) - lastpos
                 sig, nkwargs, outparam = cls._findSignature(slice=slice, packdepth=packdepth, **kwargs)
-                if state:
-                    state &= outparam.state_mask
                 slice = cls._slicecls(cls.__name__, sig, outparam, **nkwargs)
             if(lastposs):
                 slice = cls._finishSlice(slice)
@@ -177,10 +165,7 @@ class UnaryFuncDimOp(UnaryFuncOp):
         if(not found):
             raise RuntimeError, "No slice with dims to apply " + cls.__name__
         
-        if state:
-            return nslices, state      
-        else:
-            return nslices
+        return nslices
 
     @classmethod 
     def _prepareSlice(self,slice):
@@ -195,18 +180,17 @@ class UnaryFuncAggregateOp(UnaryFuncDimOp):
 class BinaryFuncOp(repops.MultiOpRep, Func):
     def __init__(self, lsource, rsource, **kwargs):
         if(not isinstance(lsource, representor.Representor)):
-            lsource = repops.PlusPrefix(wrapper_py.Rep(lsource))
+            lsource = repops.PlusPrefix(python.Rep(lsource))
         if(not isinstance(rsource, representor.Representor)):
-            rsource = repops.PlusPrefix(wrapper_py.Rep(rsource))
+            rsource = repops.PlusPrefix(python.Rep(rsource))
         repops.MultiOpRep.__init__(self,(lsource,rsource), **kwargs)
 
         
 class BinaryFuncElemOp(BinaryFuncOp):
-    _allow_partial_bc = True
+    _allow_partial_bc = False
     def _process(self, sources, **kwargs):
         lsource,rsource = sources
-        state = lsource._state & rsource._state
-        if not state & RS_TYPES_KNOWN:
+        if not lsource._typesKnown() or not rsource._typesKnown():
             return
 
         if(isinstance(lsource,repops.PlusPrefix) or isinstance(rsource,repops.PlusPrefix)):
@@ -221,12 +205,12 @@ class BinaryFuncElemOp(BinaryFuncOp):
                 pos = None
             nslices.append(self._apply(binslices, mode, pos, **kwargs))
 
-        return self._initialize(tuple(nslices),RS_CHECK)
+        return self._initialize(tuple(nslices))
 
     @classmethod
     def _apply(cls, binslices, mode, pos=None, **kwargs):
         binslices = cls._prepareSlices(*binslices)
-        (lslice,rslice),plans = ops.broadcast(binslices,mode)
+        (lslice,rslice),plans = ops.broadcast(binslices,mode, partial=cls._allow_partial_bc)
         sig, nkwargs, outparam = cls._findSignature(left=lslice,right=rslice,**kwargs)
         if(isinstance(outparam, rtypes.TypeUnknown)):
             if(binslices[0].name == binslices[1].name):
@@ -240,8 +224,7 @@ class BinaryFuncElemOp(BinaryFuncOp):
             outparam = Param(name, outparam)
         if(not pos is None):
             outparam = outparam.withNumber(pos)
-        s = ops.BinFuncElemOp(cls.__name__, sig,\
-                        outparam, allow_partial_bc=cls._allow_partial_bc, **nkwargs)
+        s = ops.BinFuncElemOp(cls.__name__, sig,outparam, **nkwargs)
         return s
 
     @classmethod
@@ -250,7 +233,6 @@ class BinaryFuncElemOp(BinaryFuncOp):
 
 
 class WithinSignature(FuncSignature):
-    _allow_partial_bc = False
     def check(self, left, right):#{{{
         in1_type = left.type
         in2_type = right.type
@@ -262,16 +244,18 @@ class WithinSignature(FuncSignature):
 within_sig = WithinSignature("within")
 
 class Within(BinaryFuncElemOp):
+    _allow_partial_bc = True
     _sigs = [within_sig]
     @classmethod
     def _prepareSlices(self,lslice,rslice):
-        return (lslice, ops.PackArrayOp(rslice,1))
+        return (lslice, ops.PackArrayOp(ops.ensure_frozen(rslice),1))
 
 class Contains(BinaryFuncElemOp):
+    _allow_partial_bc = True
     _sigs = [within_sig]
     @classmethod
     def _prepareSlices(self,lslice,rslice):
-        return (ops.PackArrayOp(lslice,1), rslice)
+        return (ops.PackArrayOp(ops.ensure_frozen(lslice),1), rslice)
 
 class BinArithSignature(FuncSignature):
     def check(self, left, right):
@@ -446,6 +430,19 @@ class GreaterEqual(BinaryFuncElemOp):
 class Greater(BinaryFuncElemOp):
     _sigs = [comparestringsig,comparesetsig,compareanysig]
 
+class MergeSignature(FuncSignature):
+    def check(self, left, right):
+        in1_type = left.type
+        in2_type = right.type
+
+        res = casts.castImplicitCommonType(in1_type, in2_type)
+        if res is False:
+            return False
+        return res#}}}
+mergesig = MergeSignature("merge")
+
+class Merge(BinaryFuncElemOp):
+    _sigs = [mergesig]
 
 class EachSignature(FuncSignature):
     def check(self, slice, eachfunc, dtype=rtypes.unknown):#{{{
