@@ -11,7 +11,7 @@ from thirdparty import tableprint, console
 
 _delay_import_(globals(),"utils","util","context","infix")
 _delay_import_(globals(),"itypes", "dimensions","dimpaths")
-_delay_import_(globals(),"wrappers","wrapper_py","wrapper")
+_delay_import_(globals(),"wrappers","python","wrapper")
 _delay_import_(globals(),"query_context")
 _delay_import_(globals(),"engines")
 _delay_import_(globals(),"repops")
@@ -35,28 +35,30 @@ class Representor(Node):
     _state = 0                 #default value
     _slices = []               #default value
 
-    def _initialize(self, slices, state=RS_ALL_KNOWN):
+    def _initialize(self, slices):
         assert isinstance(slices, tuple), "slices should be a tuple"
         self._slices = slices
 
-        if(state == RS_CHECK):
-            if(slices):
-                state |= RS_SLICES_KNOWN
-            if(all([slice.type != rtypes.unknown for slice in slices])):
-                state |= RS_TYPES_KNOWN
-            state &= ~RS_CHECK
-        self._state = state
-        
-    def _checkState(self,filter=RS_SLICES_KNOWN):
-        if(not ((self._state & filter) == filter)):
-            slices = self._getResultSlices(endpoint=False)
-            self._initialize(slices,state=RS_ALL_KNOWN | RS_INFERRED)
+    def _typesKnown(self):
+        return isinstance(self._slices, tuple) and not any([slice.type == rtypes.unknown for slice in self._slices])
+
+    def _slicesKnown(self):
+        return isinstance(self._slices,tuple)
+
+    def _checkState(self,check_type=False):
+        if(check_type is True):
+            found = self._typesKnown()
+        else:
+            found = self._slicesKnown()
+
+        if(not found):
+            self._initialize(self._getResultSlices(endpoint=False))
 
     def Show(self,table_length=100):
         print self.__str__(table_length=table_length)
 
     def __str__(self, print_data=True, table_length=15):
-        self._checkState(filter=RS_SLICES_KNOWN)
+        self._checkState()
         
         names = ["Slices:"] + [s.name for s in self._slices]
         types = ["Type:"] + [str(s.type) for s in self._slices]
@@ -219,16 +221,16 @@ class Representor(Node):
         raise AttributeError("No attribute with name: " + name + " found")
 
     def _getAttributeNames(self):
-        if not self._state & RS_SLICES_KNOWN:
+        if not self._slicesKnown():
             return []
         else:
-            print ""
-            print self.__str__(False)
+            #print ""
+            #print self.__str__(False)
             return [slice.name for slice in self._slices]
 
     def Copy(self):
         res = wrapper.SourceRepresentor()
-        res._initialize(tuple(self._getResultSlices(endpoint=False)),state=RS_CHECK)
+        res._initialize(tuple(self._getResultSlices(endpoint=False)))
         return res
 
    
@@ -237,7 +239,7 @@ class Representor(Node):
         slices = self._getResultSlices(endpoint=False)
         for slice in slices:
             slice.source = None
-        return (wrapper_py.PyRepresentor, (slices, self._state))
+        return (python.PyRepresentor, (slices,))
 
     def __copy__(self):
         nself = Representor()
@@ -514,7 +516,7 @@ class Representor(Node):
         return repops_funcs.Negative(self)
 
     def __nonzero__(self):
-        self._checkState(filter=RS_SLICES_KNOWN)
+        self._checkState()
 
         if(len(self._slices) != 1):
             raise RuntimeError, "Cannot determine True/False status of a multi-slice representor"
@@ -603,7 +605,7 @@ class Representor(Node):
         
         if(isinstance(flat,dict)):
             pass
-        elif(isinstance(flat, list)):
+        elif(isinstance(flat, (list,tuple))):
             flat = {0:flat}
         else:
             flat = {0:[flat]}
@@ -616,6 +618,13 @@ class Representor(Node):
     def Match(self, other, lslice=None,rslice=None, jointype="inner"):
         return repops_multi.Match(self, other, lslice, rslice, jointype)
 
+    def Replace(self, slice, translator, fromslice=0, toslice=1):
+        return repops_multi.Replace(self, slice, translator, fromslice, toslice) 
+
+
+    def Merge(self, other):
+        return repops_funcs.Merge(self, other)
+
     def Rename(self, *names, **kwds):
         """Rename slices.
 
@@ -669,7 +678,6 @@ class Representor(Node):
     def Each(self, eachfunc, dtype=rtypes.unknown):
         return repops_funcs.Each(self, eachfunc=eachfunc, dtype=dtype)
     
-    
     def Pos(self, dim=None):
         return repops_funcs.Pos(self, dim)
     
@@ -709,6 +717,12 @@ class Representor(Node):
     def Count(self):
         return repops_funcs.Count(self)
 
+    def __len__(self):
+        self._checkState()
+        assert len(set([s.dims[0] for s in self._slices])) == 1, "To execute len, there should be only one root dimension"
+        return self.Get(0).Array(1).Count()()
+        
+
     def Set(self, dim=None):
         return repops_funcs.Set(self,dim)
     
